#include <QRegularExpression>

#include "package.h"

Package::Package()
{

}

QList<Package> Package::Parse(const QByteArray& sdkOutput)
{
    QRegularExpression rx("\\n([^ \\n]+)\\n    Description:[\\s\\t]*([^\\n]*)\\n    Version:[\\s\\t]*([^\\n]*)\\n(    Installed Location)?");

    QList<Package> ret;

    QMap<std::pair<QString,QString>, std::pair<QString, bool>> build;

    QString str = QString::fromUtf8(sdkOutput);

    auto matches = rx.globalMatch(str);
    while(matches.hasNext()) {
        auto match = matches.next();
        if (match.hasMatch()) {

            bool isCurrentlyInstalled = (match.captured(4).length() > 0);

            auto key = std::pair<QString,QString>(match.captured(1),match.captured(3)); // PATH/VERSION
            if (build.contains(key)) {
                // maybe update CurrentlyInstalled
                if (! build[key].second && isCurrentlyInstalled) {
                    build[key].second = true;
                }

            } else {
                // seen package for the first time
                build[key] = std::pair<QString, bool>(match.captured(2), isCurrentlyInstalled); // DESCRIPTION/INSTALLED

            }
        }
    }

    // Convert deduplicated map into list for return
    for(auto i = build.begin(); i != build.end(); ++i) {

        Package p;
        p.Path = i.key().first;
        p.Description = i.value().first;
        p.Version = i.key().second;
        p.CurrentlyInstalled = i.value().second;

        ret.push_back(p);
    }

    return ret;
}
