#!/bin/bash
# Build script for Qt applications

set -eu

VERSION="$(cat ./VERSION)"
APP_NAME="$(basename $(pwd))"
PRO_NAME="${APP_NAME}.pro"
BIN_NAME="${APP_NAME}"
MXE_ROOT="/usr/lib/mxe/"

## ---

compile_windows() {
	local builddir="$(mktemp -d)"
	local srcdir="$(pwd)"
	local target="${APP_NAME}-${VERSION}-win32.7z"
	
	if [[ -f ${target} ]] ; then
		rm "${target}"
	fi
	
	(
		cd "${builddir}"
		export PATH="${MXE_ROOT}/usr/bin:${PATH}"
				
		i686-w64-mingw32.static-qmake-qt5 "VERSION+=${VERSION}" "${srcdir}/${PRO_NAME}"
		make -j6
		
		i686-w64-mingw32.static-strip "release/${BIN_NAME}.exe"		
		# UPX seems to corrupt binaries (for Qt 5.8.0 under wine, at least)
		#upx --lzma --strip-relocs=0 "release/${BIN_NAME}.exe"
		
		cd release
		#zip -9 "${target}" "${APP_NAME}.exe"
		7z a -t7z -m0=lzma -mx=9 "${target}" "${APP_NAME}.exe"
	)
	
	mv "${builddir}/release/${target}" .
	rm -r "${builddir}"
}

compile_linux() {
	local builddir="$(mktemp -d)"
	local srcdir="$(pwd)"
	local target="${APP_NAME}-${VERSION}-linux64-bin.tar.xz"
	
	if [[ -f ${target} ]] ; then
		rm "${target}"
	fi
	
	(
		cd "${builddir}"
		qmake -spec linux-g++-64 "VERSION=${VERSION}" "${srcdir}/qsdkmanager.pro"
		make -j6
		
		strip "${BIN_NAME}"
		
		tar caf "${target}" "${BIN_NAME}" --owner=0 --group=0
	)
	
	mv "${builddir}/${target}" .
	rm -r "${builddir}"
}

archive_linux() {
	local target=./${APP_NAME}-${VERSION}-src.tar.bz2
	
	if [[ -f ${target} ]] ; then
		rm "${target}"
	fi
	
	hg archive \
		--exclude _dist \
		"${target}"
}

main() {
	compile_windows
	compile_linux
	archive_linux
	
	echo "All tasks complete."
}

main "$@"

